@@ -30,6 +30,7 @@
 ;;; Code:
 ;;
 (require 'elmo)
+(require 'elmo-signal)
 (require 'elmo-msgdb)
 
 (defvar elmo-filter-number-filename "number-list"
@@ -45,8 +46,7 @@
 					    name)
   (let (pair)
     (setq pair (elmo-parse-search-condition name))
-    (elmo-filter-folder-set-condition-internal folder
-					       (car pair))
+    (elmo-filter-folder-set-condition-internal folder (car pair))
     (if (string-match "^ */\\(.*\\)$" (cdr pair))
 	(elmo-filter-folder-set-target-internal
 	 folder
@@ -57,8 +57,41 @@
      (elmo-folder-search-requires-msgdb-p
       (elmo-filter-folder-target-internal folder)
       (elmo-filter-folder-condition-internal folder)))
+    (elmo-filter-connect-signals
+     folder
+     (elmo-filter-folder-target-internal folder))
     folder))
 
+(defun elmo-filter-connect-signals (folder target)
+  (elmo-connect-signal
+   target 'flag-changing folder
+   (elmo-define-signal-handler (folder target number old-flags new-flags)
+     (elmo-filter-add-flag-count folder old-flags -1)
+     (elmo-filter-add-flag-count folder new-flags)
+     (elmo-emit-signal 'flag-changing folder number old-flags new-flags))
+   (elmo-define-signal-filter (folder target number)
+     (memq number (elmo-folder-list-messages folder nil t))))
+  (elmo-connect-signal
+   target 'flag-changed folder
+   (elmo-define-signal-handler (folder target numbers)
+     (let ((filterd (elmo-list-filter
+		     (elmo-folder-list-messages folder nil t)
+		     numbers)))
+       (when filterd
+	 (elmo-emit-signal 'flag-changed folder filterd)))))
+  (elmo-connect-signal
+   target 'cache-changed folder
+   (elmo-define-signal-handler (folder target number)
+     (elmo-emit-signal 'cache-changed folder number))
+   (elmo-define-signal-filter (folder target number)
+     (memq number (elmo-folder-list-messages folder nil t))))
+  (elmo-connect-signal
+   target 'update-overview folder
+   (elmo-define-signal-handler (folder target number)
+     (elmo-emit-signal 'update-overview folder number))
+   (elmo-define-signal-filter (folder target number)
+     (memq number (elmo-folder-list-messages folder nil t)))))
+
 (defun elmo-filter-number-list-load (dir)
   (elmo-object-load
    (expand-file-name elmo-filter-number-filename dir)))
@@ -86,6 +119,16 @@
 	  (setq flag-count (cons (cons flag delta) flag-count)))))
     (elmo-filter-folder-set-flag-count-internal folder flag-count)))
 
+(defun elmo-filter-add-flag-count (folder flags &optional delta)
+  (let ((flag-count (elmo-filter-folder-flag-count-internal folder))
+	(delta (or delta 1))
+	elem)
+    (dolist (flag flags)
+      (if (setq elem (assq flag flag-count))
+	  (setcdr elem (+ (cdr elem) delta))
+	(setq flag-count (cons (cons flag delta) flag-count))))
+    (elmo-filter-folder-set-flag-count-internal folder flag-count)))
+
 (defun elmo-filter-folder-flag-count (folder)
   (or (elmo-filter-folder-flag-count-internal folder)
       (elmo-filter-folder-countup-message-flags
@@ -112,11 +155,11 @@
 (luna-define-method elmo-folder-close-internal ((folder elmo-filter-folder))
   (elmo-folder-close-internal (elmo-filter-folder-target-internal folder)))
 
-(luna-define-method elmo-folder-close :after ((folder elmo-filter-folder))
+(luna-define-method elmo-folder-close ((folder elmo-filter-folder))
+  (elmo-generic-folder-close folder)
   (elmo-filter-folder-set-number-list-internal folder nil)
   (elmo-filter-folder-set-flag-count-internal folder nil)
-  (elmo-folder-set-msgdb-internal
-   (elmo-filter-folder-target-internal folder) nil))
+  (elmo-folder-close (elmo-filter-folder-target-internal folder)))
 
 (luna-define-method elmo-folder-commit ((folder elmo-filter-folder))
   (elmo-folder-commit (elmo-filter-folder-target-internal folder))
@@ -181,14 +224,8 @@
 (luna-define-method elmo-message-fetch ((folder elmo-filter-folder)
 					number strategy
 					&optional unseen section)
-  (unless unseen
-    (elmo-filter-folder-countup-message-flags folder (list number) -1))
-  (when (elmo-message-fetch (elmo-filter-folder-target-internal folder)
-			    number strategy unseen section)
-    (unless unseen
-      (elmo-filter-folder-countup-message-flags folder (list number))
-      (elmo-folder-notify-event folder 'flag-changed (list number)))
-    t))
+  (elmo-message-fetch (elmo-filter-folder-target-internal folder)
+		      number strategy unseen section))
 
 (luna-define-method elmo-folder-delete-messages ((folder elmo-filter-folder)
 						 numbers)
@@ -379,21 +416,15 @@
 					  numbers
 					  flag
 					  &optional is-local)
-  (elmo-filter-folder-countup-message-flags folder numbers -1)
   (elmo-folder-set-flag (elmo-filter-folder-target-internal folder)
-			numbers flag is-local)
-  (elmo-filter-folder-countup-message-flags folder numbers)
-  (elmo-folder-notify-event folder 'flag-changed numbers))
+			numbers flag is-local))
 
 (luna-define-method elmo-folder-unset-flag ((folder elmo-filter-folder)
 					    numbers
 					    flag
 					    &optional is-local)
-  (elmo-filter-folder-countup-message-flags folder numbers -1)
   (elmo-folder-unset-flag (elmo-filter-folder-target-internal folder)
-			  numbers flag is-local)
-  (elmo-filter-folder-countup-message-flags folder numbers)
-  (elmo-folder-notify-event folder 'flag-changed numbers))
+			  numbers flag is-local))
 
 (luna-define-method elmo-message-folder ((folder elmo-filter-folder)
 					 number)
