@@ -132,17 +132,13 @@
 
 (luna-define-method elmo-folder-list-subfolders ((folder elmo-flag-folder)
 						 &optional one-level)
-  (let ((dir (expand-file-name "flag" elmo-msgdb-directory)))
-    (mapcar (lambda (flag)
-	      (concat
-	       (elmo-folder-prefix-internal folder)
-	       (symbol-name (elmo-folder-type-internal folder))
-	       "/"
-	       (symbol-name flag)))
-	    (elmo-uniq-list
-	     (append
-	      (mapcar 'intern (delete ".." (delete "." (directory-files dir))))
-	      (copy-sequence elmo-global-flags))))))
+  (mapcar (lambda (flag)
+	    (concat
+	     (elmo-folder-prefix-internal folder)
+	     (symbol-name (elmo-folder-type-internal folder))
+	     "/"
+	     (symbol-name flag)))
+	  elmo-global-flags))
 
 (defun elmo-flag-folder-delete-message (folder number
 					       &optional keep-referrer)
@@ -232,7 +228,7 @@
 						    flag
 						    &optional is-local)
   (when (eq flag (elmo-flag-folder-flag-internal folder))
-    (error "Cannot unset flag `%s' in this folder." flag)))
+    (error "Cannot unset flag `%s' in this folder" flag)))
 
 ;;; Utilities
 
@@ -303,7 +299,7 @@ NUMBER is the message number."
 
 (defsubst elmo-global-flag-set-internal (flag folder number message-id)
   (when (elmo-local-flag-p flag)
-    (error "Cannot treat `%s' as global flag." flag))
+    (error "Cannot treat `%s' as global flag" flag))
   (when message-id
     (let ((flag-folder (elmo-flag-get-folder flag))
 	  cache new-file new-number elem)
@@ -445,6 +441,18 @@ If optional IGNORE-PRESERVED is non-nil, preserved flags
 	(setq result (delq flag result))))
     result))
 
+(defun elmo-global-flags-initialize (&optional additional-flags)
+  (let ((dir (expand-file-name "flag" elmo-msgdb-directory)))
+    (setq elmo-global-flags
+	  (elmo-list-delete
+	   elmo-local-flags
+	   (elmo-uniq-list
+	    (append
+	     elmo-global-flags
+	     additional-flags
+	     (mapcar 'intern
+		     (delete ".." (delete "." (directory-files dir))))))))))
+
 ;;; To migrate from global mark folder
 (defvar elmo-global-mark-filename "global-mark"
   "Obsolete variable. (Just for migration)")
@@ -454,7 +462,7 @@ If optional IGNORE-PRESERVED is non-nil, preserved flags
   (when (and (file-exists-p (expand-file-name elmo-global-mark-filename
 					      elmo-msgdb-directory))
 	     (elmo-global-flag-p 'important)
-	     (not (file-exists-p (elmo-folder-expand-msgdb-path
+	     (not (file-exists-p (elmo-folder-msgdb-path
 				  (elmo-flag-get-folder 'important)))))
     (elmo-global-mark-upgrade)))
 
