@@ -31,6 +31,7 @@
 (eval-when-compile (require 'cl))
 
 (require 'elmo)
+(require 'elmo-signal)
 (require 'luna)
 
 (defvar elmo-multi-divide-number 100000
@@ -67,8 +68,58 @@
   (elmo-multi-folder-set-divide-number-internal
    folder
    elmo-multi-divide-number)
+  (elmo-multi-connect-signals folder)
   folder)
 
+(defun elmo-multi-connect-signals (folder)
+  (elmo-connect-signal
+   nil 'flag-changing folder
+   (elmo-define-signal-handler (folder child number old-flags new-flags)
+     (elmo-emit-signal 'flag-changing folder
+		       (car (elmo-multi-map-numbers folder child (list number)))
+		       old-flags new-flags))
+   (elmo-define-signal-filter (folder sender)
+     (memq sender (elmo-multi-folder-children-internal folder))))
+  (elmo-connect-signal
+   nil 'flag-changed folder
+   (elmo-define-signal-handler (folder child numbers)
+     (elmo-emit-signal 'flag-changed folder
+		       (elmo-multi-map-numbers folder child numbers)))
+   (elmo-define-signal-filter (folder sender)
+     (memq sender (elmo-multi-folder-children-internal folder))))
+  (elmo-connect-signal
+   nil 'cache-changed folder
+   (elmo-define-signal-handler (folder child number)
+     (elmo-emit-signal
+      'flag-changed folder
+      (car (elmo-multi-map-numbers folder child (list number)))))
+   (elmo-define-signal-filter (folder sender)
+     (memq sender (elmo-multi-folder-children-internal folder))))
+  (elmo-connect-signal
+   nil 'update-overview folder
+   (elmo-define-signal-handler (folder child number)
+     (elmo-emit-signal
+      'update-overview folder
+      (car (elmo-multi-map-numbers folder child (list number)))))
+   (elmo-define-signal-filter (folder sender)
+     (memq sender (elmo-multi-folder-children-internal folder)))))
+
+(defun elmo-multi-map-numbers (folder child numbers)
+  (let ((multi (catch 'found
+		 (let ((children (elmo-multi-folder-children-internal folder))
+		       (index 0))
+		   (while children
+		     (setq index (1+ index))
+		     (when (eq (car children) child)
+		       (throw 'found index))
+		     (setq children (cdr children)))))))
+    (when multi
+      (let ((offset (* (elmo-multi-folder-divide-number-internal folder)
+		       multi)))
+      (mapcar (lambda (number) (+ offset number))
+	      numbers)))))
+
+
 (luna-define-method elmo-folder-open-internal ((folder elmo-multi-folder))
   (dolist (fld (elmo-multi-folder-children-internal folder))
     (elmo-folder-open-internal fld)))
@@ -81,9 +132,10 @@
   (dolist (fld (elmo-multi-folder-children-internal folder))
     (elmo-folder-close-internal fld)))
 
-(luna-define-method elmo-folder-close :after ((folder elmo-multi-folder))
+(luna-define-method elmo-folder-close ((folder elmo-multi-folder))
+  (elmo-generic-folder-close folder)
   (dolist (fld (elmo-multi-folder-children-internal folder))
-    (elmo-folder-set-msgdb-internal fld nil)))
+    (elmo-folder-close fld)))
 
 (luna-define-method elmo-folder-synchronize ((folder elmo-multi-folder)
 					     &optional
@@ -246,11 +298,7 @@
 					number strategy
 					&optional unseen section)
   (let ((pair (elmo-multi-real-folder-number folder number)))
-    (when (elmo-message-fetch (car pair) (cdr pair)
-			      strategy unseen section)
-      (unless unseen
-	(elmo-folder-notify-event folder 'flag-changed (list number)))
-      t)))
+    (elmo-message-fetch (car pair) (cdr pair) strategy unseen section)))
 
 (luna-define-method elmo-folder-delete-messages ((folder elmo-multi-folder)
 						 numbers)
@@ -270,26 +318,24 @@
   (elmo-multi-folder-diff folder))
 
 (defun elmo-multi-folder-diff (folder)
-  (let ((flds (elmo-multi-folder-children-internal folder))
-	(news 0)
+  (let ((news 0)
 	(unreads 0)
 	(alls 0)
-	no-unreads diff)
-    (while flds
-      (setq diff (elmo-folder-diff (car flds)))
-      (cond
-       ((consp (cdr diff)) ; (new unread all)
-	(setq news    (+ news (nth 0 diff))
-	      unreads (+ unreads (nth 1 diff))
-	      alls    (+ alls (nth 2 diff))))
-       (t
-	(setq no-unreads t)
-	(setq news    (+ news (car diff))
-	      alls    (+ alls (cdr diff)))))
-      (setq flds (cdr flds)))
-    (if no-unreads
-	(cons news alls)
-      (list news unreads alls))))
+	diff value)
+    (dolist (child (elmo-multi-folder-children-internal folder))
+      (setq diff (elmo-folder-diff child))
+      (setq news    (and news
+			 (setq value (elmo-diff-new diff))
+			 (+ news value))
+	    unreads (and unreads
+			 (setq value (elmo-diff-unread diff))
+			 (+ unreads value))
+	    alls    (and alls
+			 (setq value (elmo-diff-all diff))
+			 (+ alls value))))
+    (if unreads
+	(list news unreads alls)
+      (cons news alls))))
 
 (luna-define-method elmo-folder-list-messages
   ((folder elmo-multi-folder) &optional visible-only in-msgdb)
@@ -426,8 +472,7 @@
 					  flag
 					  &optional is-local)
   (dolist (pair (elmo-multi-make-folder-numbers-list folder numbers))
-    (elmo-folder-set-flag (car pair) (cdr pair) flag is-local))
-  (elmo-folder-notify-event folder 'flag-changed numbers))
+    (elmo-folder-set-flag (car pair) (cdr pair) flag is-local)))
 
 (luna-define-method elmo-folder-unset-flag ((folder elmo-multi-folder)
 					    numbers
@@ -435,8 +480,7 @@
 					    &optional is-local)
   (dolist (pair (elmo-multi-make-folder-numbers-list folder numbers))
     (ignore-errors
-     (elmo-folder-unset-flag (car pair) (cdr pair) flag is-local)))
-  (elmo-folder-notify-event folder 'flag-changed numbers))
+     (elmo-folder-unset-flag (car pair) (cdr pair) flag is-local))))
 
 (luna-define-method elmo-folder-list-flagged ((folder elmo-multi-folder)
 					      flag
