@@ -162,22 +162,21 @@ If it is the symbol `all', update overview for all shimbun folders."
 	(setq shimbun-id (elmo-message-entity-field entity 'message-id))
       (setq message-id (elmo-message-entity-field entity 'message-id)
 	    shimbun-id nil))
-    (elmo-set-work-buf
-     (set-buffer-multibyte t)
-     (shimbun-make-header
-      (elmo-message-entity-number entity)
-      (shimbun-mime-encode-string
-       (elmo-message-entity-field entity 'subject 'decode))
-      (shimbun-mime-encode-string
-       (elmo-message-entity-field entity 'from 'decode))
-      (elmo-message-entity-field entity 'date)
-      message-id
-      (elmo-message-entity-field entity 'references)
-      0
-      0
-      (elmo-message-entity-field entity 'xref)
-      (and shimbun-id
-	   (list (cons "x-shimbun-id" shimbun-id)))))))
+    (elmo-with-enable-multibyte
+      (shimbun-make-header
+       (elmo-message-entity-number entity)
+       (shimbun-mime-encode-string
+	(elmo-message-entity-field entity 'subject 'decode))
+       (shimbun-mime-encode-string
+	(elmo-message-entity-field entity 'from 'decode))
+       (elmo-message-entity-field entity 'date)
+       message-id
+       (elmo-message-entity-field entity 'references)
+       0
+       0
+       (elmo-message-entity-field entity 'xref)
+       (and shimbun-id
+	    (list (cons "x-shimbun-id" shimbun-id)))))))
 
 (defsubst elmo-shimbun-folder-header-hash-setup (folder headers)
   (let ((hash (or (elmo-shimbun-folder-header-hash-internal folder)
@@ -378,43 +377,34 @@ If it is the symbol `all', update overview for all shimbun folders."
   (let ((entity (elmo-message-entity folder shimbun-id))
 	(message-id (shimbun-header-id header))
 	references)
-    (unless (string= shimbun-id message-id)
-      (elmo-message-entity-set-field
-       entity 'x-original-id message-id)
-      (elmo-shimbun-header-set-extra-field
-       header "x-shimbun-id" shimbun-id)
-      (elmo-set-hash-val message-id
-			 entity
-			 (elmo-shimbun-folder-entity-hash folder))
-      (elmo-set-hash-val shimbun-id
-			 entity
-			 (elmo-shimbun-folder-entity-hash folder)))
-    (elmo-message-entity-set-field
-     entity
-     'from
-     (elmo-mime-string (shimbun-header-from header)))
-    (elmo-message-entity-set-field
-     entity
-     'subject
-     (elmo-mime-string (shimbun-header-subject header)))
-    (elmo-message-entity-set-field
-     entity
-     'date
-     (shimbun-header-date header))
-    (when (setq references
-		(or (elmo-msgdb-get-last-message-id
-		     (elmo-field-body "in-reply-to"))
-		    (elmo-msgdb-get-last-message-id
-		     (elmo-field-body "references"))))
-      (elmo-message-entity-set-field
-       entity
-       'references
-       (or (elmo-message-entity-field
-	    (elmo-get-hash-val
-	     references
-	     (elmo-shimbun-folder-entity-hash folder))
-	    'message-id)
-	   references)))))
+    (when (elmo-msgdb-update-entity
+	   (elmo-folder-msgdb folder)
+	   entity
+	   (nconc
+	    (unless (string= shimbun-id message-id)
+	      (elmo-shimbun-header-set-extra-field
+	       header "x-shimbun-id" shimbun-id)
+	      (elmo-set-hash-val message-id
+				 entity
+				 (elmo-shimbun-folder-entity-hash folder))
+	      (elmo-set-hash-val shimbun-id
+				 entity
+				 (elmo-shimbun-folder-entity-hash folder))
+	      (list (cons 'x-original-id message-id)))
+	    (list
+	     (cons 'from
+		   (elmo-mime-string (shimbun-header-from header)))
+	     (cons 'subject
+		   (elmo-mime-string (shimbun-header-subject header)))
+	     (cons 'date
+		   (shimbun-header-date header))
+	     (cons 'references
+		   (or (elmo-msgdb-get-last-message-id
+			(elmo-field-body "in-reply-to"))
+		       (elmo-msgdb-get-last-message-id
+			(elmo-field-body "references")))))))
+      (elmo-emit-signal 'update-overview folder
+			(elmo-message-entity-number entity)))))
 
 (luna-define-method elmo-map-message-fetch ((folder elmo-shimbun-folder)
 					    location strategy
@@ -514,11 +504,12 @@ If it is the symbol `all', update overview for all shimbun folders."
 
 (luna-define-method elmo-folder-exists-p ((folder elmo-shimbun-folder))
   (if (elmo-shimbun-folder-group-internal folder)
-      (progn
+      (if (fboundp 'shimbun-group-p)
+	  (shimbun-group-p (elmo-shimbun-folder-shimbun-internal folder)
+			   (elmo-shimbun-folder-group-internal folder))
 	(member
 	 (elmo-shimbun-folder-group-internal folder)
-	 (shimbun-groups (elmo-shimbun-folder-shimbun-internal
-			  folder))))
+	 (shimbun-groups (elmo-shimbun-folder-shimbun-internal folder))))
     t))
 
 (luna-define-method elmo-folder-delete-messages ((folder elmo-shimbun-folder)
@@ -526,6 +517,13 @@ If it is the symbol `all', update overview for all shimbun folders."
   (elmo-folder-kill-messages folder numbers)
   t)
 
+(luna-define-method elmo-message-entity-parent ((folder elmo-shimbun-folder)
+						entity)
+  (let ((references (elmo-message-entity-field entity 'references)))
+    (and references
+	 (elmo-get-hash-val references
+			    (elmo-shimbun-folder-entity-hash folder)))))
+
 (require 'product)
 (product-provide (provide 'elmo-shimbun) (require 'elmo-version))
 
