@@ -88,24 +88,34 @@
 (def-edebug-spec elmo-bind-directory
   (form &rest form))
 
+(defconst elmo-multibyte-buffer-name " *elmo-multibyte-buffer*")
+
+(defmacro elmo-with-enable-multibyte (&rest body)
+  "Evaluate BODY with `default-enable-multibyte-character'."
+  `(with-current-buffer (get-buffer-create elmo-multibyte-buffer-name)
+     ,@body))
+
+(put 'elmo-with-enable-multibyte 'lisp-indent-function 0)
+(def-edebug-spec elmo-with-enable-multibyte t)
+
 (defun elmo-object-load (filename &optional mime-charset no-err)
   "Load OBJECT from the file specified by FILENAME.
 File content is decoded with MIME-CHARSET."
-    (if (not (file-readable-p filename))
-	nil
-      (elmo-set-work-buf
-       (as-binary-input-file
-	(insert-file-contents filename))
-       (when mime-charset
-	 (set-buffer-multibyte default-enable-multibyte-characters)
-	 (decode-mime-charset-region (point-min) (point-max) mime-charset))
-       (condition-case nil
-	   (read (current-buffer))
-	 (error (unless no-err
-		  (message "Warning: Loading object from %s failed."
-			   filename)
-		  (elmo-object-save filename nil))
-		nil)))))
+  (if (not (file-readable-p filename))
+      nil
+    (with-temp-buffer
+      (as-binary-input-file
+       (insert-file-contents filename))
+      (when mime-charset
+	(set-buffer-multibyte default-enable-multibyte-characters)
+	(decode-mime-charset-region (point-min) (point-max) mime-charset))
+      (condition-case nil
+	  (read (current-buffer))
+	(error (unless no-err
+		 (message "Warning: Loading object from %s failed."
+			  filename)
+		 (elmo-object-save filename nil))
+	       nil)))))
 
 (defsubst elmo-save-buffer (filename &optional mime-charset)
   "Save current buffer to the file specified by FILENAME.
@@ -129,11 +139,11 @@ File content is encoded with MIME-CHARSET."
   "Save OBJECT to the file specified by FILENAME.
 Directory of the file is created if it doesn't exist.
 File content is encoded with MIME-CHARSET."
-  (elmo-set-work-buf
-   (let (print-length print-level)
-     (prin1 object (current-buffer)))
-;;;(princ "\n" (current-buffer))
-   (elmo-save-buffer filename mime-charset)))
+  (with-temp-buffer
+    (let (print-length print-level)
+      (prin1 object (current-buffer)))
+;;;    (princ "\n" (current-buffer))
+    (elmo-save-buffer filename mime-charset)))
 
 ;;; Search Condition
 
@@ -727,9 +737,34 @@ the directory becomes empty after deletion."
       (setq list (cdr list)))
     ret-val))
 
-(defun elmo-list-diff (list1 list2 &optional mes)
-  (if mes
-      (message "%s" mes))
+(defun elmo-list-diff (list1 list2)
+  (let ((clist1 (sort (copy-sequence list1) #'<))
+	(clist2 (sort (copy-sequence list2) #'<))
+	list1-only list2-only)
+    (while (or clist1 clist2)
+      (cond
+       ((null clist1)
+	(while clist2
+	  (setq list2-only (cons (car clist2) list2-only))
+	  (setq clist2 (cdr clist2))))
+       ((null clist2)
+	(while clist1
+	  (setq list1-only (cons (car clist1) list1-only))
+	  (setq clist1 (cdr clist1))))
+       ((< (car clist1) (car clist2))
+	(while (and clist1 (< (car clist1) (car clist2)))
+	  (setq list1-only (cons (car clist1) list1-only))
+	  (setq clist1 (cdr clist1))))
+       ((< (car clist2) (car clist1))
+	(while (and clist2 (< (car clist2) (car clist1)))
+	  (setq list2-only (cons (car clist2) list2-only))
+	  (setq clist2 (cdr clist2))))
+       ((= (car clist1) (car clist2))
+	(setq clist1 (cdr clist1)
+	      clist2 (cdr clist2)))))
+    (list list1-only list2-only)))
+
+(defun elmo-list-diff-nonsortable (list1 list2)
   (let ((clist1 (copy-sequence list1))
 	(clist2 (copy-sequence list2)))
     (while list2
@@ -738,8 +773,6 @@ the directory becomes empty after deletion."
     (while list1
       (setq clist2 (delq (car list1) clist2))
       (setq list1 (cdr list1)))
-    (if mes
-	(message "%sdone" mes))
     (list clist1 clist2)))
 
 (defun elmo-list-bigger-diff (list1 list2 &optional mes)
@@ -774,13 +807,13 @@ the directory becomes empty after deletion."
   `(or (vectorp ,filter) (consp ,filter)))
 
 (defmacro elmo-filter-type (filter)
-  (` (aref (, filter) 0)))
+  `(aref ,filter 0))
 
 (defmacro elmo-filter-key (filter)
-  (` (aref (, filter) 1)))
+  `(aref ,filter 1))
 
 (defmacro elmo-filter-value (filter)
-  (` (aref (, filter) 2)))
+  `(aref ,filter 2))
 
 (defsubst elmo-buffer-field-primitive-condition-match (condition
 						       number
@@ -831,25 +864,6 @@ the directory becomes empty after deletion."
 	(setq result (not result)))
     result))
 
-(defun elmo-condition-in-msgdb-p-internal (condition fields)
-  (cond
-   ((vectorp condition)
-    (if (not (member (elmo-filter-key condition) fields))
-	(throw 'found t)))
-   ((or (eq (car condition) 'and)
-	(eq (car condition) 'or))
-    (elmo-condition-in-msgdb-p-internal (nth 1 condition) fields)
-    (elmo-condition-in-msgdb-p-internal (nth 2 condition) fields))))
-
-(defun elmo-condition-in-msgdb-p (condition)
-  (not (catch 'found
-	 (elmo-condition-in-msgdb-p-internal condition
-					     (append
-					      elmo-msgdb-extra-fields
-					      '("last" "first" "from"
-						"subject" "to" "cc" "since"
-						"before"))))))
-
 (defun elmo-buffer-field-condition-match (condition number number-list)
   (cond
    ((vectorp condition)
@@ -866,51 +880,6 @@ the directory becomes empty after deletion."
 	(elmo-buffer-field-condition-match
 	 (nth 2 condition) number number-list)))))
 
-(defsubst elmo-file-field-primitive-condition-match (file
-						     condition
-						     number
-						     number-list)
-  (let (result)
-    (goto-char (point-min))
-    (cond
-     ((string= (elmo-filter-key condition) "last")
-      (setq result (<= (length (memq number number-list))
-		       (string-to-int (elmo-filter-value condition))))
-      (if (eq (elmo-filter-type condition) 'unmatch)
-	  (setq result (not result))))
-     ((string= (elmo-filter-key condition) "first")
-      (setq result (< (- (length number-list)
-			 (length (memq number number-list)))
-		      (string-to-int (elmo-filter-value condition))))
-      (if (eq (elmo-filter-type condition) 'unmatch)
-	  (setq result (not result))))
-     (t
-      (elmo-set-work-buf
-       (as-binary-input-file (insert-file-contents file))
-       (set-buffer-multibyte default-enable-multibyte-characters)
-       ;; Should consider charset?
-       (decode-mime-charset-region (point-min)(point-max) elmo-mime-charset)
-       (setq result
-	     (elmo-buffer-field-primitive-condition-match
-	      condition number number-list)))))
-    result))
-
-(defun elmo-file-field-condition-match (file condition number number-list)
-  (cond
-   ((vectorp condition)
-    (elmo-file-field-primitive-condition-match
-     file condition number number-list))
-   ((eq (car condition) 'and)
-    (and (elmo-file-field-condition-match
-	  file (nth 1 condition) number number-list)
-	 (elmo-file-field-condition-match
-	  file (nth 2 condition) number number-list)))
-   ((eq (car condition) 'or)
-    (or (elmo-file-field-condition-match
-	 file (nth 1 condition) number number-list)
-	(elmo-file-field-condition-match
-	 file (nth 2 condition) number number-list)))))
-
 (defmacro elmo-get-hash-val (string hashtable)
   (static-if (fboundp 'unintern)
       `(symbol-value (intern-soft ,string ,hashtable))
@@ -953,15 +922,10 @@ Emacs 19.28 or earlier does not have `unintern'."
 (defsubst elmo-mime-string (string)
   "Normalize MIME encoded STRING."
   (and string
-       (elmo-set-work-buf
-	(set-buffer-multibyte default-enable-multibyte-characters)
-	(setq string
-	      (encode-mime-charset-string
-	       (eword-decode-and-unfold-unstructured-field-body
-		string)
-	       elmo-mime-charset))
-	(set-buffer-multibyte nil)
-	string)))
+       (elmo-with-enable-multibyte
+	 (encode-mime-charset-string
+	  (eword-decode-and-unfold-unstructured-field-body string)
+	  elmo-mime-charset))))
 
 (defsubst elmo-collect-field (beg end downcase-field-name)
   (save-excursion
@@ -1224,8 +1188,8 @@ SPEC is a list as followed (LABEL MAX-VALUE [FORMAT])."
 (defun elmo-decoded-field-body (field-name &optional mode)
   (let ((field-body (elmo-field-body field-name)))
     (and field-body
-	 (elmo-set-work-buf
-	  (mime-decode-field-body field-body field-name mode)))))
+	 (elmo-with-enable-multibyte
+	   (mime-decode-field-body field-body field-name mode)))))
 
 (defun elmo-address-quote-specials (word)
   "Make quoted string of WORD if needed."
