@@ -327,7 +327,7 @@ When non-nil, redundunt message-id string are not saved."
 	   new-flags diff)
        (unless (memq flag cur-flags)
 	 (setq new-flags (cons flag cur-flags))
-	 (setq diff (elmo-list-diff new-flags cur-flags))
+	 (setq diff (elmo-list-diff-nonsortable new-flags cur-flags))
 	 (modb-standard-countup-flags msgdb (car diff))
 	 (modb-standard-countup-flags msgdb (cadr diff) -1)
 	 (elmo-set-hash-val (modb-standard-key number)
@@ -350,10 +350,11 @@ When non-nil, redundunt message-id string are not saved."
 			  (modb-standard-flag-map msgdb)))
     (t
      (let ((cur-flags (modb-standard-message-flags msgdb number))
+	   (inhibit-quit t)
 	   new-flags diff)
        (when (memq flag cur-flags)
 	 (setq new-flags (delq flag (copy-sequence cur-flags)))
-	 (setq diff (elmo-list-diff new-flags cur-flags))
+	 (setq diff (elmo-list-diff-nonsortable new-flags cur-flags))
 	 (modb-standard-countup-flags msgdb (car diff))
 	 (modb-standard-countup-flags msgdb (cadr diff) -1)
 	 (elmo-set-hash-val (modb-standard-key number)
@@ -475,6 +476,15 @@ When non-nil, redundunt message-id string are not saved."
 	  (modb-standard-set-flag-modified msgdb number))
 	duplicate))))
 
+(luna-define-method elmo-msgdb-update-entity ((msgdb modb-standard)
+					      entity values)
+  (let ((handler (elmo-message-entity-handler entity)))
+    (when (elmo-msgdb-message-entity-update-fields handler entity values)
+      (modb-standard-set-message-modified
+       msgdb
+       (elmo-msgdb-message-entity-number handler entity))
+      t)))
+
 (luna-define-method elmo-msgdb-delete-messages ((msgdb modb-standard)
 						numbers)
   (let ((number-list (modb-standard-number-list-internal msgdb))
