@@ -632,7 +632,8 @@
 See info under Wanderlust for full documentation.
 
 Special commands:
-\\{wl-draft-mode-map}"))
+\\{wl-draft-mode-map}"
+    (setq font-lock-defaults nil)))
 
 (defun wl-draft-key-setup ()
   (define-key wl-draft-mode-map "\C-c\C-y" 'wl-draft-yank-original)
