@@ -1,6 +1,6 @@
 ;;; wl-highlight.el --- Hilight modules for Wanderlust.
 
-;; Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+;; Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
 ;;  Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
@@ -189,9 +189,8 @@
 
 ;; important messages
 
-(wl-defface wl-highlight-summary-important-face
-  '(
-    (((type tty)
+(wl-defface wl-highlight-summary-flagged-face
+  '((((type tty)
       (background dark))
      (:foreground "magenta"))
     (((class color)
@@ -200,7 +199,7 @@
     (((class color)
       (background light))
      (:foreground "purple")))
-  "Face used for displaying important messages."
+  "Face used for displaying flagged messages."
   :group 'wl-summary-faces
   :group 'wl-faces)
 
@@ -250,7 +249,7 @@
   :group 'wl-summary-faces
   :group 'wl-faces)
 
-(wl-defface wl-highlight-summary-deleted-face
+(wl-defface wl-highlight-summary-disposed-face
   '(
     (((type tty)
       (background dark))
@@ -261,10 +260,55 @@
     (((class color)
       (background light))
      (:foreground "DarkKhaki")))
+  "Face used for displaying messages mark as disposed."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
+(wl-defface wl-highlight-summary-deleted-face
+  '(
+    (((type tty)
+      (background dark))
+     (:foreground "blue"))
+    (((class color)
+      (background dark))
+     (:foreground "SteelBlue"))
+    (((class color)
+      (background light))
+     (:foreground "RoyalBlue4")))
   "Face used for displaying messages mark as deleted."
   :group 'wl-summary-faces
   :group 'wl-faces)
 
+(wl-defface wl-highlight-summary-prefetch-face
+  '(
+    (((type tty)
+      (background dark))
+     (:foreground "Green"))
+    (((class color)
+      (background dark))
+     (:foreground "DeepSkyBlue"))
+    (((class color)
+      (background light))
+     (:foreground "brown")))
+  "Face used for displaying messages mark as deleted."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
+(wl-defface wl-highlight-summary-resend-face
+  '(
+    (((type tty)
+      (background dark))
+     (:foreground "Yellow"))
+    (((class color)
+      (background dark))
+     (:foreground "orange3"))
+    (((class color)
+      (background light))
+     (:foreground "orange3")))
+  "Face used for displaying messages mark as resend."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
 (wl-defface wl-highlight-summary-refiled-face
   '(
     (((type tty)
@@ -295,6 +339,49 @@
   :group 'wl-summary-faces
   :group 'wl-faces)
 
+;; answered
+(wl-defface wl-highlight-summary-answered-face
+  '((((type tty)
+      (background dark))
+     (:foreground "yellow"))
+    (((class color)
+      (background dark))
+     (:foreground "khaki"))
+    (((class color)
+      (background light))
+     (:foreground "khaki4")))
+  "Face used for displaying answered messages."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
+;; forwarded
+(wl-defface wl-highlight-summary-forwarded-face
+  '((((type tty)
+      (background dark))
+     (:foreground "yellow"))
+    (((class color)
+      (background dark))
+     (:foreground "DarkOliveGreen2"))
+    (((class color)
+      (background light))
+     (:foreground "DarkOliveGreen4")))
+  "Face used for displaying forwarded messages."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
+(wl-defface wl-summary-persistent-mark-face
+  '((((type tty))
+     (:foreground "blue"))
+    (((class color)
+      (background dark))
+     (:foreground "SeaGreen4"))
+    (((class color)
+      (background light))
+     (:foreground "SeaGreen1")))
+  "Dafault face used for displaying messages with persistent mark."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
 ;; obsolete.
 (wl-defface wl-highlight-summary-temp-face
   '(
@@ -534,41 +621,38 @@
   :group 'wl-faces)
 
 (wl-defface wl-highlight-demo-face
-  '(
-    (((type tty)
-      (background dark))
+  '((((type tty))
      (:foreground "green"))
-    (((class color)
-      (background dark))
-     (:foreground "GreenYellow"))
     (((class color)
       (background light))
-     (:foreground "blue2")))
+     (:foreground "#006600" :background "#d9ffd9"))
+    (((class color)
+      (background dark))
+     (:foreground "#d9ffd9" :background "#004400")))
   "Face used for displaying demo."
   :group 'wl-faces)
 
 (wl-defface wl-highlight-logo-face
-  '(
-    (((type tty)
+  '((((type tty)
       (background dark))
      (:foreground "cyan"))
-    (((class color)
-      (background dark))
-     (:foreground "SkyBlue"))
     (((class color)
       (background light))
-     (:foreground "SteelBlue")))
+     (:foreground "SteelBlue" :background "#d9ffd9"))
+    (((class color)
+      (background dark))
+     (:foreground "SkyBlue" :background "#004400")))
   "Face used for displaying demo."
   :group 'wl-faces)
 
-(wl-defface wl-highlight-refile-destination-face
+(wl-defface wl-highlight-action-argument-face
   '((((class color)
       (background dark))
      (:foreground "pink"))
     (((class color)
       (background light))
      (:foreground "red")))
-  "Face used for displaying refile destination."
+  "Face used for displaying action argument."
   :group 'wl-summary-faces
   :group 'wl-faces)
 
@@ -688,19 +772,20 @@
   :group 'wl-message-faces
   :group 'wl-faces)
 
+(defface wl-message-header-narrowing-face
+  '((((class color) (background light))
+     (:foreground "black" :background "dark khaki"))
+    (((class color) (background dark))
+     (:foreground "white" :background "dark goldenrod"))
+    (t (:bold t)))
+  "Face used for header narrowing for the message."
+  :group 'wl-message-faces
+  :group 'wl-faces)
+
 (defvar wl-highlight-folder-opened-regexp " *\\(\\[\\-\\]\\)")
 (defvar wl-highlight-folder-closed-regexp " *\\(\\[\\+\\]\\)")
 (defvar wl-highlight-folder-leaf-regexp "[ ]*\\([-%\\+]\\)\\(.*\\):.*$")
 
-(defvar wl-highlight-summary-unread-regexp " *[0-9]+[^0-9]\\(!\\|U\\)")
-(defvar wl-highlight-summary-important-regexp " *[0-9]+[^0-9]\\$")
-(defvar wl-highlight-summary-new-regexp " *[0-9]+[^0-9]N")
-(defvar wl-highlight-summary-deleted-regexp " *[0-9]+D")
-(defvar wl-highlight-summary-refiled-regexp " *[0-9]+o")
-(defvar wl-highlight-summary-copied-regexp " *[0-9]+O")
-(defvar wl-highlight-summary-target-regexp " *[0-9]+\\*")
-;;(defvar wl-highlight-summary-thread-top-regexp " *[0-9]+[^0-9][^0-9]../..\(.*\)..:.. \\[")
-
 (defvar wl-highlight-citation-face-list
   '(wl-highlight-message-cited-text-1
     wl-highlight-message-cited-text-2
@@ -769,121 +854,127 @@
 	      (put-text-property bol (match-end 0) 'face face)))
 	(put-text-property bol eol 'face text-face)))))
 
-(defun wl-highlight-summary-line-string (line mark temp-mark indent)
-  (let (fsymbol)
-    (cond ((and (string= temp-mark wl-summary-score-over-mark)
-		(member mark (list wl-summary-unread-cached-mark
-				   wl-summary-unread-uncached-mark
-				   wl-summary-new-mark)))
-	   (setq fsymbol 'wl-highlight-summary-high-unread-face))
-	  ((and (string= temp-mark wl-summary-score-below-mark)
-		(member mark (list wl-summary-unread-cached-mark
-				   wl-summary-unread-uncached-mark
-				   wl-summary-new-mark)))
-	   (setq fsymbol 'wl-highlight-summary-low-unread-face))
-	  ((string= temp-mark "o")
-	   (setq fsymbol 'wl-highlight-summary-refiled-face))
-	  ((string= temp-mark "O")
-	   (setq fsymbol 'wl-highlight-summary-copied-face))
-	  ((string= temp-mark "D")
-	   (setq fsymbol 'wl-highlight-summary-deleted-face))
-	  ((string= temp-mark "*")
-	   (setq fsymbol 'wl-highlight-summary-temp-face))
-	  ((string= mark wl-summary-new-mark)
-	   (setq fsymbol 'wl-highlight-summary-new-face))
-	  ((member mark (list wl-summary-unread-cached-mark
-			      wl-summary-unread-uncached-mark))
-	   (setq fsymbol 'wl-highlight-summary-unread-face))
-	  ((or (string= mark wl-summary-important-mark))
-	   (setq fsymbol 'wl-highlight-summary-important-face))
-	  ((string= temp-mark wl-summary-score-below-mark)
-	   (setq fsymbol 'wl-highlight-summary-low-read-face))
-	  ((string= temp-mark wl-summary-score-over-mark)
-	   (setq fsymbol 'wl-highlight-summary-high-read-face))
-	  (t (if (zerop (length indent))
-		 (setq fsymbol 'wl-highlight-summary-thread-top-face)
-	       (setq fsymbol 'wl-highlight-summary-normal-face))))
-    (put-text-property 0 (length line) 'face fsymbol line))
-  (if wl-use-highlight-mouse-line
-      (put-text-property 0 (length line) 'mouse-face 'highlight line)))
+(defsubst wl-highlight-get-face-by-name (format &rest args)
+  (let ((face (intern (apply #'format format args))))
+    (and (find-face face)
+	 face)))
 
-(defun wl-highlight-summary-current-line (&optional smark regexp temp-too)
+(defsubst wl-highlight-summary-line-face-spec (flags temp-mark indent)
+  "Return a cons cell of (face . argument)."
+  (let (action)
+    (if (setq action (assoc temp-mark wl-summary-mark-action-list))
+	(cons (nth 5 action) (nth 2 action))
+      (cond
+       ((and (string= temp-mark wl-summary-score-over-mark)
+	     (or (memq 'new flags) (memq 'unread flags)))
+	'(wl-highlight-summary-high-unread-face))
+       ((and (string= temp-mark wl-summary-score-below-mark)
+	     (or (memq 'new flags) (memq 'unread flags)))
+	'(wl-highlight-summary-low-unread-face))
+       ((let ((priorities wl-summary-persistent-mark-priority-list)
+	      (fl wl-summary-flag-alist)
+	      face result global-flags)
+	  (while (and (null result) priorities)
+	    (if (eq (car priorities) 'flag)
+		(when (setq global-flags
+			    (elmo-get-global-flags flags 'ignore-preserved))
+		  (while fl
+		    (when (memq (car (car fl)) global-flags)
+		      (setq result
+			    (list (or (wl-highlight-get-face-by-name
+				       "wl-highlight-summary-%s-flag-face"
+				       (car (car fl)))
+				      'wl-highlight-summary-flagged-face))
+			    fl nil))
+		    (setq fl (cdr fl)))
+		  (unless result
+		    (setq result (list 'wl-highlight-summary-flagged-face))))
+	      (when (memq (car priorities) flags)
+		(setq result
+		      (list (or (wl-highlight-get-face-by-name
+				 "wl-highlight-summary-%s-face"
+				 (car priorities))
+				'wl-summary-persistent-mark-face)))))
+	    (setq priorities (cdr priorities)))
+	  result))
+       ((string= temp-mark wl-summary-score-below-mark)
+	'(wl-highlight-summary-low-read-face))
+       ((string= temp-mark wl-summary-score-over-mark)
+	'(wl-highlight-summary-high-read-face))
+       (t (if indent
+	      '(wl-highlight-summary-normal-face)
+	    '(wl-highlight-summary-thread-top-face)))))))
+
+(autoload 'elmo-flag-folder-referrer "elmo-flag")
+(defun wl-highlight-flag-folder-help-echo (folder number)
+  (let ((referer (elmo-flag-folder-referrer folder number)))
+    (concat "The message exists in "
+	    (mapconcat
+	     (lambda (pair)
+		   (concat (car pair) "/"
+			   (number-to-string
+			    (cdr pair))))
+	     referer ","))))
+
+(defun wl-highlight-summary-line-help-echo (number beg end &optional string)
+  (let ((type (elmo-folder-type-internal wl-summary-buffer-elmo-folder))
+	message handler)
+    (when (setq handler (cadr (assq type wl-highlight-summary-line-help-echo-alist)))
+      (setq message
+	    (funcall handler wl-summary-buffer-elmo-folder number))
+      (if message
+	  (put-text-property beg end 'help-echo
+			     message
+			     string)))))
+
+(defun wl-highlight-summary-line-string (number line flags temp-mark indent)
+  (let ((fsymbol (car (wl-highlight-summary-line-face-spec
+		       flags
+		       temp-mark
+		       (> (length indent) 0)))))
+    (put-text-property 0 (length line) 'face fsymbol line))
+  (when wl-use-highlight-mouse-line
+    (put-text-property 0 (length line) 'mouse-face 'highlight line))
+  (when wl-highlight-summary-line-help-echo-alist
+    (wl-highlight-summary-line-help-echo number 0 (length line) line)))
+
+(defun wl-highlight-summary-current-line (&optional number flags)
   (interactive)
   (save-excursion
     (let ((inhibit-read-only t)
-	  (case-fold-search nil) temp-mark status-mark
+	  (case-fold-search nil)
 	  (deactivate-mark nil)
-	  fsymbol bol eol matched thread-top looked-at dest ds)
-      (end-of-line)
-      (setq eol (point))
-      (beginning-of-line)
-      (setq bol (point))
-      (if smark
-	  (setq status-mark smark)
-	(setq status-mark (wl-summary-persistent-mark)))
-      (when temp-too
-	(setq temp-mark (wl-summary-temp-mark))
-	(cond
-	 ((string= temp-mark "*")
-	  (setq fsymbol 'wl-highlight-summary-temp-face))
-	 ((string= temp-mark "D")
-	  (setq fsymbol 'wl-highlight-summary-deleted-face))
-	 ((string= temp-mark "O")
-	  (setq fsymbol 'wl-highlight-summary-copied-face
-		dest t))
-	 ((string= temp-mark "o")
-	  (setq fsymbol 'wl-highlight-summary-refiled-face
-		dest t))))
-      (if (not fsymbol)
-	  (cond
-	   ((and (string= temp-mark wl-summary-score-over-mark)
-		 (member status-mark (list wl-summary-unread-cached-mark
-					   wl-summary-unread-uncached-mark
-					   wl-summary-new-mark)))
-	    (setq fsymbol 'wl-highlight-summary-high-unread-face))
-	   ((and (string= temp-mark wl-summary-score-below-mark)
-		 (member status-mark (list wl-summary-unread-cached-mark
-					   wl-summary-unread-uncached-mark
-					   wl-summary-new-mark)))
-	    (setq fsymbol 'wl-highlight-summary-low-unread-face))
-	   ((string= status-mark wl-summary-new-mark)
-	    (setq fsymbol 'wl-highlight-summary-new-face))
-	   ((member status-mark (list wl-summary-unread-cached-mark
-				      wl-summary-unread-uncached-mark))
-	    (setq fsymbol 'wl-highlight-summary-unread-face))
-	   ((string= status-mark wl-summary-important-mark)
-	    (setq fsymbol 'wl-highlight-summary-important-face))
-	   ;; score mark
-	   ((string= temp-mark wl-summary-score-below-mark)
-	    (setq fsymbol 'wl-highlight-summary-low-read-face))
-	   ((string= temp-mark wl-summary-score-over-mark)
-	    (setq fsymbol 'wl-highlight-summary-high-read-face))
-	   ;;
-	   (t (if (null
-		   (wl-thread-entity-get-parent-entity
-		    (wl-thread-get-entity (wl-summary-message-number))))
-		  (setq fsymbol 'wl-highlight-summary-thread-top-face)
-		(setq fsymbol 'wl-highlight-summary-normal-face)))))
-      (put-text-property bol eol 'face fsymbol)
-      (when dest
-	(put-text-property (next-single-property-change
-			    (next-single-property-change
-			     bol 'wl-summary-destination
-			     nil eol)
-			    'wl-summary-destination nil eol)
-			   eol
-			   'face
-			   'wl-highlight-refile-destination-face))
-      (if wl-use-highlight-mouse-line
-	  (put-text-property bol
-;;; Use bol instead of (1- (match-end 0))
-;;;			     (1- (match-end 0))
-			     eol 'mouse-face 'highlight))
-;;;   (put-text-property (match-beginning 3) (match-end 3)
-;;;			 'face 'wl-highlight-thread-indent-face)
-      ;; Dnd stuff.
-      (if wl-use-dnd
-	  (wl-dnd-set-drag-starter bol eol)))))
+	  (number (or number (wl-summary-message-number)))
+	  bol eol spec)
+      (when number
+	(end-of-line)
+	(setq eol (point))
+	(beginning-of-line)
+	(setq bol (point))
+	(setq spec (wl-highlight-summary-line-face-spec
+		    (or flags
+			(elmo-message-flags wl-summary-buffer-elmo-folder
+					    number))
+		    (wl-summary-temp-mark number)
+		    (wl-thread-entity-get-parent-entity
+		     (wl-thread-get-entity number))))
+	(when (car spec)
+	  (put-text-property bol eol 'face (car spec)))
+	(when (cdr spec)
+	  (put-text-property (next-single-property-change
+			      (next-single-property-change
+			       bol 'wl-summary-action-argument
+			       nil eol)
+			      'wl-summary-action-argument nil eol)
+			     eol
+			     'face
+			     'wl-highlight-action-argument-face))
+	(when wl-use-highlight-mouse-line
+	  (put-text-property bol eol 'mouse-face 'highlight))
+	(when wl-highlight-summary-line-help-echo-alist
+	  (wl-highlight-summary-line-help-echo number bol eol))
+	(when wl-use-dnd
+	  (wl-dnd-set-drag-starter bol eol))))))
 
 (defun wl-highlight-folder (start end)
   "Highlight folder between start and end.
@@ -939,9 +1030,9 @@ Variables used:
 	  (overlay-put ov 'wl-momentary-overlay t))
 	(forward-line 1)))))
 
-(defun wl-highlight-refile-destination-string (string)
+(defun wl-highlight-action-argument-string (string)
   (put-text-property 0 (length string) 'face
-		     'wl-highlight-refile-destination-face
+		     'wl-highlight-action-argument-face
 		     string))
 
 (defun wl-highlight-summary-all ()
@@ -953,20 +1044,11 @@ Variables used:
   "Highlight summary between start and end.
 Faces used:
   wl-highlight-summary-unread-face      unread messages
-  wl-highlight-summary-important-face   important messages
   wl-highlight-summary-deleted-face     messages mark as deleted
   wl-highlight-summary-refiled-face     messages mark as refiled
   wl-highlight-summary-copied-face      messages mark as copied
   wl-highlight-summary-new-face         new messages
-
-Variables used:
-  wl-highlight-summary-unread-regexp    matches unread messages
-  wl-highlight-summary-important-regexp matches important messages
-  wl-highlight-summary-deleted-regexp   matches messages mark as deleted
-  wl-highlight-summary-refiled-regexp   matches messages mark as refiled
-  wl-highlight-summary-copied-regexp    matches messages mark as copied
-  wl-highlight-summary-new-regexp       matches new messages
-"
+  wl-highlight-summary-*-flag-face      flagged messages"
   (if (< end start)
       (let ((s start)) (setq start end end s)))
   (let (lines too-big gc-message e p hend i percent)
@@ -981,9 +1063,7 @@ Variables used:
 		  (< (point) end))
 	(when (or (not lazy)
 		  (null (get-text-property (point) 'face)))
-	  (wl-highlight-summary-current-line nil nil
-					     (or wl-summary-lazy-highlight
-						 wl-summary-scored)))
+	  (wl-highlight-summary-current-line))
 	(forward-line 1))
       (unless wl-summary-lazy-highlight
 	(message "Highlighting...done")))))
@@ -1003,7 +1083,6 @@ This function is defined for `window-scroll-functions'"
 				'lazy))
 	(set-buffer-modified-p nil)))))
 
-
 (defun wl-highlight-headers (&optional for-draft)
   (let ((beg (point-min))
 	(end (or (save-excursion (re-search-forward "^$" nil t)
@@ -1046,17 +1125,25 @@ Returns start point of signature."
     (goto-char end)
     (or
      ;; look for legal signature separator (check at first for fasten)
-     (re-search-backward "\n-- \n" beg t)
+     (search-backward "\n-- \n" beg t)
 
      ;; look for dual separator
-     (save-excursion
-       (and
-	(re-search-backward "^[^A-Za-z0-9> \t\n]+ *$" beg t)
-	(> (- (match-end 0) (match-beginning 0)) 10);; "10" is a magic number.
-	(re-search-backward
-	 (concat "^"
-		 (regexp-quote (buffer-substring (match-beginning 0) (match-end 0)))
-		 "$") beg t)))
+     (let ((pt (point))
+	   separator)
+       (prog1
+	   (and (re-search-backward "^[^A-Za-z0-9> \t\n]+ *$" beg t)
+		;; `10' is a magic number.
+		(> (- (match-end 0) (match-beginning 0)) 10)
+		(setq separator (buffer-substring (match-beginning 0)
+						  (match-end 0)))
+		;; We should not use `re-search-backward' for a long word
+		;; since it is possible to crash XEmacs because of a bug.
+		(if (search-backward (concat "\n" separator "\n") beg t)
+		    (1+ (point))
+		  (and (search-backward (concat separator "\n") beg t)
+		       (bolp)
+		       (point))))
+	 (goto-char pt)))
 
      ;; look for user specified signature-separator
      (if (stringp wl-highlight-signature-separator)
@@ -1073,23 +1160,23 @@ Returns start point of signature."
 Faces used:
   wl-highlight-message-headers			  the part before the colon
   wl-highlight-message-header-contents		  the part after the colon
-  wl-highlight-message-important-header-contents  contents of \"special\"
+  wl-highlight-message-important-header-contents  contents of \"important\"
                                                   headers
-  wl-highlight-message-important-header-contents2 contents of \"special\"
+  wl-highlight-message-important-header-contents2 contents of \"important\"
                                                   headers
   wl-highlight-message-unimportant-header-contents contents of unimportant
                                                    headers
-  wl-highlight-message-cited-text	           quoted text from other
+  wl-highlight-message-cited-text-N	           quoted text from other
                                                    messages
   wl-highlight-message-citation-header             header of quoted texts
   wl-highlight-message-signature                   signature
 
 Variables used:
-  wl-highlight-important-header-regexp	 what makes a \"special\" header
-  wl-highlight-important-header2-regexp	 what makes a \"special\" header
-  wl-highlight-unimportant-header-regexp what makes a \"special\" header
-  wl-highlight-citation-prefix-regexp	 matches lines of quoted text
-  wl-highlight-citation-header-regexp	 matches headers for quoted text
+  wl-highlight-message-header-alist             alist of header regexp with
+                                                face for header contents
+  wl-highlight-citation-prefix-regexp		matches lines of quoted text
+  wl-highlight-force-citation-header-regexp	matches headers for quoted text
+  wl-highlight-citation-header-regexp		matches headers for quoted text
 
 If HACK-SIG is true,then we search backward from END for something that
 looks like the beginning of a signature block, and don't consider that a
@@ -1122,7 +1209,7 @@ interpreted as cited text.)"
 	    ;; If this search fails then the narrowing performed above
 	    ;; is sufficient
 	    (if (re-search-forward (format
-				    "^$\\|%s"
+				    "^\\(%s\\)?$"
 				    (regexp-quote mail-header-separator))
 				   nil t)
 		(narrow-to-region (point-min) (match-beginning 0)))
@@ -1132,33 +1219,31 @@ interpreted as cited text.)"
 	      (goto-char start)
 	      (while (and (not body-only)
 			  (not (eobp)))
-		(cond
-		 ((looking-at "^[^ \t\n:]+[ \t]*:")
-		  (put-text-property (match-beginning 0) (match-end 0)
-				     'face 'wl-highlight-message-headers)
-		  (setq p (match-end 0))
-		  (setq hend (save-excursion (std11-field-end end)))
-		  (cond
-		   ((catch 'match
-		      (let ((regexp-alist wl-highlight-message-header-alist))
-			(while regexp-alist
-			  (when (save-match-data
-				  (looking-at (caar regexp-alist)))
-			    (put-text-property p hend 'face
-					       (cdar regexp-alist))
-			    (throw 'match t))
-			  (setq regexp-alist (cdr regexp-alist)))
-			(throw 'match nil))))
-		   (t
-		    (put-text-property
-		     p hend 'face 'wl-highlight-message-header-contents)))
-		  (goto-char hend))
-		 ;; ignore non-header field name lines
-		 (t (forward-line 1))))))
+		(if (looking-at "^[^ \t\n:]+[ \t]*:[ \t]*")
+		    (progn
+		      (put-text-property (match-beginning 0) (match-end 0)
+					 'face 'wl-highlight-message-headers)
+		      (setq p (match-end 0))
+		      (setq hend (save-excursion (std11-field-end end)))
+		      (or (catch 'match
+			    (let ((regexp-alist wl-highlight-message-header-alist))
+			      (while regexp-alist
+				(when (save-match-data
+					(looking-at (caar regexp-alist)))
+				  (put-text-property p hend 'face
+						     (cdar regexp-alist))
+				  (throw 'match t))
+				(setq regexp-alist (cdr regexp-alist)))
+			      (throw 'match nil)))
+			  (put-text-property
+			   p hend 'face 'wl-highlight-message-header-contents))
+		      (goto-char hend))
+		  ;; ignore non-header field name lines
+		  (forward-line 1)))))
 	  (let (prefix prefix-face-alist pair end)
 	    (while (not (eobp))
 	      (cond
-	       ((looking-at mail-header-separator)
+	       ((looking-at (concat "^" (regexp-quote mail-header-separator) "$"))
 		(put-text-property (match-beginning 0) (match-end 0)
 				   'face 'wl-highlight-header-separator-face)
 		(goto-char (match-end 0)))
@@ -1217,6 +1302,7 @@ interpreted as cited text.)"
 	 (inhibit-read-only t))
     (put-text-property beg end 'mouse-face 'highlight)))
 
+
 (require 'product)
 (product-provide (provide 'wl-highlight) (require 'wl-version))
 
