@@ -57,9 +57,11 @@ Special commands:
 	  (start (progn (beginning-of-line) (point)))
 	  (inhibit-read-only t)
 	  (text-face
-	   (cond ((looking-at wl-highlight-folder-opened-regexp)
+	   (cond ((and (wl-folder-buffer-group-p)
+		       (looking-at wl-highlight-folder-opened-regexp))
 		  'wl-highlight-folder-opened-face)
-		 ((looking-at wl-highlight-folder-closed-regexp)
+		 ((and (wl-folder-buffer-group-p)
+		       (looking-at wl-highlight-folder-closed-regexp))
 		  'wl-highlight-folder-closed-face)
 		 (t
 		  (if (looking-at (format "^[ \t]*\\(%s\\|%s\\)"
@@ -102,6 +104,9 @@ Special commands:
   (let ((keymap (make-sparse-keymap)))
     (define-key keymap "D" 'wl-message-delete-current-part)
     (define-key keymap "l" 'wl-message-toggle-disp-summary)
+    (define-key keymap "\C-c:d" 'wl-message-decrypt-pgp-nonmime)
+    (define-key keymap "\C-c:v" 'wl-message-verify-pgp-nonmime)
+    (define-key keymap "w" 'wl-draft)
     (define-key keymap [mouse-4] 'wl-message-wheel-down)
     (define-key keymap [mouse-5] 'wl-message-wheel-up)
     (define-key keymap [S-mouse-4] 'wl-message-wheel-down)
@@ -161,7 +166,9 @@ Special commands:
   (define-key wl-draft-mode-map "\C-c\C-a" 'wl-addrmgr)
   (define-key wl-draft-mode-map "\C-xk" 'wl-draft-mimic-kill-buffer)
   (define-key wl-draft-mode-map "\C-c\C-d" 'wl-draft-elide-region)
-  (define-key wl-draft-mode-map "\C-a" 'wl-draft-beginning-of-line))
+  (define-key wl-draft-mode-map "\C-a" 'wl-draft-beginning-of-line)
+  (define-key wl-draft-mode-map "\M-p" 'wl-draft-previous-history-element)
+  (define-key wl-draft-mode-map "\M-n" 'wl-draft-next-history-element))
 
 (defun wl-draft-overload-menubar ()
   (let ((keymap (current-local-map)))
@@ -301,6 +308,14 @@ Warning: Unknown req `%S' with options `%S'" req options)
   (let ((event (read-event)))
     (cons (and (numberp event) event) event)))
 
+(defun wl-completing-read-multiple (prompt
+				    table
+				    &optional predicate
+				    require-match initial-input)
+  "Read multiple strings in the minibuffer"
+  (split-string (completing-read prompt table predicate require-match
+				 initial-input) ","))
+
 (require 'product)
 (product-provide (provide 'wl-mule) (require 'wl-version))
 
