@@ -90,10 +90,10 @@
      wl-summary-jump-to-current-message t "Jump to Current Message"]
     [wl-summary-sync-force-update
      wl-summary-sync-force-update t "Sync Current Folder"]
-    [wl-summary-delete
-     wl-summary-delete t "Delete Current Message"]
-    [wl-summary-mark-as-important
-     wl-summary-mark-as-important t "Mark Current Message as Important"]
+    [wl-summary-dispose
+     wl-summary-dispose t "Dispose Current Message"]
+    [wl-summary-set-flags
+     wl-summary-set-flags t "Set Flags"]
     [wl-draft
      wl-summary-write-current-folder t "Write for Current Folder"]
     [wl-summary-reply
@@ -253,12 +253,14 @@
     (let (fld-name)
       (cond
        (;; opened folder group
-	(looking-at wl-highlight-folder-opened-regexp)
+	(and (wl-folder-buffer-group-p)
+	     (looking-at wl-highlight-folder-opened-regexp))
 	(wl-xmas-highlight-folder-group-line 'wl-folder-opened-glyph
 					     'wl-highlight-folder-opened-face
 					     numbers))
        (;; closed folder group
-	(looking-at wl-highlight-folder-closed-regexp)
+	(and (wl-folder-buffer-group-p)
+	     (looking-at wl-highlight-folder-closed-regexp))
 	(wl-xmas-highlight-folder-group-line 'wl-folder-closed-glyph
 					     'wl-highlight-folder-closed-face
 					     numbers))
@@ -289,7 +291,9 @@
 		((string= fld-name wl-queue-folder);; queue folder
 		 (get 'wl-folder-queue-glyph 'glyph))
 		(;; and one of many other folders
-		 (setq type (elmo-folder-type fld-name))
+		 (setq type (or (elmo-folder-type fld-name)
+				(elmo-folder-type-internal
+				 (elmo-make-folder fld-name))))
 		 (get (intern (format "wl-folder-%s-glyph" type)) 'glyph))))))
 	  (let ((end (point-at-eol)))
 	    (when wl-use-highlight-mouse-line
@@ -378,6 +382,7 @@
     (wl-folder-maildir-glyph	. wl-maildir-folder-icon)
     (wl-folder-nmz-glyph	. wl-nmz-folder-icon)
     (wl-folder-shimbun-glyph	. wl-shimbun-folder-icon)
+    (wl-folder-file-glyph	. wl-file-folder-icon)
     (wl-folder-trash-empty-glyph . wl-empty-trash-folder-icon)
     (wl-folder-draft-glyph	. wl-draft-folder-icon)
     (wl-folder-queue-glyph	. wl-queue-folder-icon)
@@ -449,6 +454,9 @@
   (let ((keymap (make-sparse-keymap)))
     (define-key keymap "D" 'wl-message-delete-current-part)
     (define-key keymap "l" 'wl-message-toggle-disp-summary)
+    (define-key keymap "\C-c:d" 'wl-message-decrypt-pgp-nonmime)
+    (define-key keymap "\C-c:v" 'wl-message-verify-pgp-nonmime)
+    (define-key keymap "w" 'wl-draft)
     (define-key keymap 'button4 'wl-message-wheel-down)
     (define-key keymap 'button5 'wl-message-wheel-up)
     (define-key keymap [(shift button4)] 'wl-message-wheel-down)
@@ -533,7 +541,9 @@ Special commands:
   (define-key wl-draft-mode-map "\C-c\C-a" 'wl-addrmgr)
   (define-key wl-draft-mode-map "\C-xk"    'wl-draft-mimic-kill-buffer)
   (define-key wl-draft-mode-map "\C-c\C-d" 'wl-draft-elide-region)
-  (define-key wl-draft-mode-map "\C-a" 'wl-draft-beginning-of-line))
+  (define-key wl-draft-mode-map "\C-a" 'wl-draft-beginning-of-line)
+  (define-key wl-draft-mode-map "\M-p" 'wl-draft-previous-history-element)
+  (define-key wl-draft-mode-map "\M-n" 'wl-draft-next-history-element))
 
 (defun wl-draft-overload-functions ()
   (wl-mode-line-buffer-identification)
@@ -556,6 +566,14 @@ Special commands:
 	       (event-to-character event))
 	  event)))
 
+(defun wl-completing-read-multiple (prompt
+				    table
+				    &optional predicate
+				    require-match initial-input)
+  "Read multiple strings in the minibuffer"
+  (split-string (completing-read prompt table predicate require-match
+				 initial-input) ","))
+
 (require 'product)
 (product-provide (provide 'wl-xmas) (require 'wl-version))
 
